using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Ivi.Visa.Interop;

namespace Measurements_5314x
{
    public partial class MainForm : Form
    {
        private Ivi.Visa.Interop.FormattedIO488 counter;

        public MainForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            // Add initial values to list box
            listBox1.Items.Add("Enter/select instrument address, if needed,");
            listBox1.Items.Add("click on \"Select I/O\" to check/validate the address,");
            listBox1.Items.Add("and click on \"Get Readings\" to trigger instrument.");
            listBox1.Items.Add("Measurements will display after a few seconds delay.");

            // if no function, initialize to Freq 1
            if (funcSelect.SelectedIndex < 0)
                funcSelect.SelectedIndex = 0;

        }

        private void SelectIO_Click(object sender, EventArgs e)
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // Button that selects the I/O and creates an instrument session.
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            OpenPort();
        }

        public class Globls
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // Sets global variables to be used in different functions of the program.
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            public static bool connected = false;  // Used to determine if there is connection with the instrument
            public static string myAddr;		// Used to hold IO address
            public static double NumRdgs;		// Used for the number of readings taken
            public static string unit1;
            public static string unit2;
        }

        public void End_Prog()
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // Closes the program
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            if (Globls.connected)
                // Close the session
                counter.IO.Close();


        }

        public bool OpenPort()
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // This function opens a port (the communication between the instrument and
        // computer).
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            string addr,
                    msg,
                    option_string;

            Globls.connected = false;

            try
            {
                // create the formatted IO object
                counter = new FormattedIO488Class();

                //create the resource manager
                ResourceManager mgr = new ResourceManager();

                // Get the address
                addr = (ioAddr.Text).ToUpper();
                if (addr.Length < 8)
                {
                    MessageBox.Show("Incorrect address; check address", "Measurements_5314x");
                    ioAddr.Text = "GPIB0::22";
                    ioAddr.Refresh();
                    Globls.connected = false;
                    goto Exit_Func;
                }
                else if (addr.IndexOf("GPIB") < 0)
                {
                    MessageBox.Show("Only GPIB interface supported; check address", "Measurements_5314x");
                    ioAddr.Text = "GPIB0::22";
                    ioAddr.Refresh();
                    Globls.connected = false;
                    goto Exit_Func;
                }

                // if user did not end string with ::INSTR, then do so
                if (addr.IndexOf("::INSTR") < 0)
                {
                    //append ::INSTR to address
                    addr = addr + "::INSTR";
                }

                Globls.myAddr = addr;

                // OK, we tried to make sure we got good address, but still can fail
                // Try to open the device, 5 second time out
                option_string = "";
                counter.IO = (IMessage)mgr.Open(addr, AccessMode.NO_LOCK, 5000, option_string);

                // Check and make sure the correct instrument is addressed
                counter.WriteString("*IDN?", true);
                msg = counter.ReadString();

                // Remove cr and or lf character
                msg = msg.Remove((msg.Length - 2), 2);

                // Enter ID string into list box so user can see it
                // This will be helpful in case ID verification fails below
                listBox1.Items.Clear();
                listBox1.Items.Add("Instrument ID is:");
                listBox1.Items.Add(msg);
                
                // Verify ID, if not 5314x counter then error and return
                if ((msg.IndexOf("53147") < 0) && (msg.IndexOf("53148") < 0) && (msg.IndexOf("53149") < 0))
                {
                    MessageBox.Show("Instrument was not a 53147, 53148, or 53149 counter.", "Measurements_5314x");
                    ioAddr.Text = "GPIB0::22";
                    ioAddr.Refresh();
                    Globls.connected = false;
                    goto Exit_Func;
                }

                // if we got here, then we are connectd to a 5314x
                Globls.connected = true;

            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message + "\nin function: OpenPort. Likely due to bad IO address. \nVerify address field and press Select I/O button.",
                    "Measurements_5314x");
                goto Exit_Func;
            }

        Exit_Func:
            return Globls.connected;

        }


        public void Check_Error(string msg)
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // Checks for syntax and other errors.
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            int err_num;
            bool exit_code = false;
            string err_msg, show_msg;

            // Check for exceptions			
            try
            {
                // Read error queue
                counter.WriteString("SYSTem:ERRor?", true);
                err_msg = counter.ReadString();

                // Get error number and message
                err_num = Convert.ToInt16(err_msg.Substring(0, err_msg.IndexOf(",")));
                err_msg = err_msg.Substring((err_msg.IndexOf(",") + 1), (err_msg.Length - (Convert.ToInt16(err_msg.IndexOf(","))) - 1));

                // If error found, check for more errors and exit program
                while (err_num != 0)
                {
                    exit_code = true;

                    show_msg = "Error in: \"" + msg + "\" function\n";
                    show_msg += "Error: " + err_num.ToString() + "," + err_msg;

                    MessageBox.Show(show_msg, "Measurements_5314x");

                    // Read error queue
                    counter.WriteString("SYSTem:ERRor?", true);
                    err_msg = counter.ReadString();

                    // Get error number and message
                    err_num = Convert.ToInt16(err_msg.Substring(0, err_msg.IndexOf(",")));
                    err_msg = err_msg.Substring((err_msg.IndexOf(",") + 1), (err_msg.Length - (Convert.ToInt16(err_msg.IndexOf(","))) - 1));
                }

                if (exit_code)
                {
                    counter.WriteString("*CLS", true);

                    End_Prog();
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message + "\nin function: CheckError", "Measurements_5314x");
                End_Prog();
            }

        }

        public void Readings()
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // This function triggers the instrument and takes readings.
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            string  reading,	// Used to store reading
                    reading2;   // Second reading, if Freq & Pow selected
            int ii;			    // Loop counter
 //           object[] rdgs = new object[3]; // Returns reading from memory

            Cursor savedCursor = this.Cursor;
            this.Cursor = Cursors.WaitCursor;

            // Check for exceptions			
            try
            {
                // Trigger instrument
                counter.WriteString("INIT:CONT ON", true);
                System.Threading.Thread.Sleep(3000);  // wait 3 seconds
                for (ii = 0; ii < 5; ii++)
                {
                    if (funcSelect.SelectedIndex < 2)
                    {
                        // only one reading to grab
                        counter.WriteString("FETC:FREQ?", true);
                        reading = counter.ReadString();
                        // Remove cr and or lf character
                        reading = reading.Remove((reading.Length - 1), 1);
                        listBox1.Items.Add((ii + 1).ToString() + "\t" + reading);
                    }
                    else if (funcSelect.SelectedIndex == 2)
                    {
                        // Power
                        // only one reading to grab
                        counter.WriteString("FETC:POW?", true);
                        reading = counter.ReadString();
                        // Remove cr and or lf character
                        reading = reading.Remove((reading.Length - 1), 1);
                        listBox1.Items.Add((ii + 1).ToString() + "\t" + reading);
                    }
                    else if (funcSelect.SelectedIndex == 3)
                    {
                        // Voltage
                        // only one reading to grab
                        counter.WriteString("FETC:VOLT?", true);
                        reading = counter.ReadString();
                        // Remove cr and or lf character
                        reading = reading.Remove((reading.Length - 1), 1);
                        listBox1.Items.Add((ii + 1).ToString() + "\t" + reading);
                    }
                    else
                    {
                        // is Freq and Power
                        counter.WriteString("FETC:FREQ?", true);
                        reading = counter.ReadString();
                        // Remove cr and or lf character
                        reading = reading.Remove((reading.Length - 1), 1);
                        counter.WriteString("FETC:POW?", true);
                        reading2 = counter.ReadString();
                        // Remove cr and or lf character
                        reading2 = reading2.Remove((reading2.Length - 1), 1);
                        listBox1.Items.Add((ii + 1).ToString() + "\t" + reading + "\t" + reading2);
                    }

                    Application.DoEvents();
 
                    // delay before fetching next reading(s)
                    System.Threading.Thread.Sleep(600);  // wait 600 mS

                }

                this.Cursor = savedCursor;

            }
            catch (Exception e)
            {
                this.Cursor = savedCursor;

                MessageBox.Show(e.Message + "\nin function: Readings", "VISAComExample");
                End_Prog();
            }
        }
        
        public void SetupMeas()
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        // This function performs the instrument setup.
        //"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
        {
            string msg;

            // Check for exceptions			
            try
            {
                // Could be measurement in progress from previous, so ABORt
                counter.WriteString("ABOR;*CLS;*RST", true);

                // Wait for Reset to settle
                counter.WriteString("*OPC?", true);
                string respStr = counter.ReadString();

                // determine which measurement we will be making
                switch (funcSelect.SelectedIndex)
                {
                    case -1:
                    case 0:
                        // do Freq 1 if unselected or Freq 1
                        listBox1.Items.Add("Reading" + "\t" + " Frequency Hz");
                        listBox1.Items.Add("================================================");
                        counter.WriteString(":CONF:FREQ DEF,DEF,(@1)", true);
                        break;

                    case 1:
                        // Freq 2 selected
                        listBox1.Items.Add("Reading" + "\t" + "Frequency Hz");
                        listBox1.Items.Add("================================================");
                        counter.WriteString(":CONF:FREQ DEF,DEF,(@2)", true);
                        break;

                    case 2:
                        // Power selected
                        listBox1.Items.Add("Reading" + "\t" + "Power");
                        listBox1.Items.Add("==================================");
                        counter.WriteString(":CONF:POW;:UNIT:POW DBM", true);
                        break;

                    case 3:
                        // Voltage
                        listBox1.Items.Add("Reading" + "\t" + "Voltage");
                        listBox1.Items.Add("==================================");
                        counter.WriteString(":CONF:VOLT", true);
                        break;

                    case 4:
                        // Freq 1 and Power selected
                        listBox1.Items.Add("Reading" + "\t" + "Frequency Hz" + "\t" + "Power");
                        listBox1.Items.Add("===========================================================");
                        counter.WriteString(":CONF:FREQ DEF,DEF,(@1)", true);
                        counter.WriteString(":CONF:POW;:UNIT:POW DBM", true);
                        break;

                    case 5:
                        // Freq 2 and Power selected
                        listBox1.Items.Add("Reading" + "\t" + "Frequency Hz" + "\t" + "Power");
                        listBox1.Items.Add("===========================================================");
                        counter.WriteString(":CONF:FREQ DEF,DEF,(@2)", true);
                        counter.WriteString(":CONF:POW;:UNIT:POW DBM", true);
                        break;

                    default:
                        // do Freq 1 if we get here
                        listBox1.Items.Add("Reading" + "\t" + "Frequency Hz");
                        listBox1.Items.Add("================================================");
                        counter.WriteString(":CONF:FREQ DEF,DEF,(@1)", true);
                        break;

                } // end switch(funcSelect.SelectedIndex)

                // pause here to make sure instrument is done
                counter.WriteString("*OPC?", true);
                respStr = counter.ReadString();

                //Check for errors
                Check_Error("SetupMeas");

            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message + "\nin function: Setup", "Measurements_5314x");
                End_Prog();
            }
        }


        public void RunProg()
        // Executes the main loop to set up and then take readings
        {
            // if not already connected, call OpenPort
            if (!Globls.connected)
                if (!OpenPort())
                    return;      // return if can't open

            // Setup for selected measurement
            SetupMeas();

            // Call routine to do the readings
            Readings();

        }

        private void EndProgram_Click(object sender, EventArgs e)
        {
            End_Prog();
            //Environment.Exit(0);
            Application.Exit();
        }

        private void btnTakeReadings_Click(object sender, EventArgs e)
        {
            // clear the list box
            listBox1.Items.Clear();
            
            // Start the measurement
            RunProg();

        }
    }
}